require 'pathname'

Puppet::Type.newtype(:dsc_xexchantimalwarescanning) do
  require Pathname.new(__FILE__).dirname + '../../' + 'puppet/type/base_dsc'

  provide :powershell, :parent => Puppet::Type.type(:base_dsc).provider(:powershell) do
    defaultfor :operatingsystem => :windows
  end

  @doc = %q{
    The DSC xExchAntiMalwareScanning resource type.
    Originally generated from the following schema.mof file:
      import/dsc_resources/xExchange/DSCResources/MSFT_xExchAntiMalwareScanning/MSFT_xExchAntiMalwareScanning.schema.mof
  }

  validate do
      fail('dsc_enabled is a required attribute') if self[:dsc_enabled].nil?
    end

  newparam(:dscmeta_resource_friendly_name) do
    defaultto "xExchAntiMalwareScanning"
  end

  newparam(:dscmeta_resource_name) do
    defaultto "MSFT_xExchAntiMalwareScanning"
  end

  newparam(:dscmeta_import_resource) do
    newvalues(true, false)

    munge do |value|
      provider.munge_boolean(value.to_s)
    end

    defaultto true
  end

  newparam(:dscmeta_module_name) do
    defaultto "xExchange"
  end

  newparam(:dscmeta_module_version) do
    defaultto "1.2.0.0"
  end

  newparam(:name, :namevar => true ) do
  end

  ensurable do
    newvalue(:exists?) { provider.exists? }
    newvalue(:present) { provider.create }
    defaultto { :present }
  end

  # Name:         Enabled
  # Type:         boolean
  # IsMandatory:  True
  # Values:       None
  newparam(:dsc_enabled) do
    desc "Whether Exchange Anti-malware scanning should be Enabled"
    isrequired
    validate do |value|
    end
    newvalues(true, false)
    munge do |value|
      provider.munge_boolean(value.to_s)
    end
  end

  # Name:         Credential
  # Type:         string
  # IsMandatory:  False
  # Values:       None
  newparam(:dsc_credential) do
    desc "Credentials used to establish a remote Powershell session to Exchange"
    validate do |value|
      unless value.kind_of?(String)
        fail("Invalid value '#{value}'. Should be a string")
      end
    end
  end

  # Name:         AllowServiceRestart
  # Type:         boolean
  # IsMandatory:  False
  # Values:       None
  newparam(:dsc_allowservicerestart) do
    desc "Whether the Transport services should be automatically restarted after a status change"
    validate do |value|
    end
    newvalues(true, false)
    munge do |value|
      provider.munge_boolean(value.to_s)
    end
  end


end
